#pragma once

//==============================================================================
template <typename Type>
class MyCustomDspOscillator
{
public:
    //==============================================================================
    MyCustomDspOscillator()
    {
        auto& osc = OscillatorProcessorChain.template get<oscIndex>();
        
        int waveform = 1;
        
        if (waveform == 0){
            osc.initialise ([] (Type x)
                            {
                                return jmap (x, Type (-MathConstants<double>::pi), Type (MathConstants<double>::pi), Type (-1), Type (1));
                            }, 2);
        }else
        {
            osc.initialise ([] (Type x) { return std::sin (x); }, 128);
        }

        
        osc.setFrequency(432.0f, false);
        
        auto& gain = OscillatorProcessorChain.template get<gainIndex>();
        gain.setGainLinear(0.015); //gain.setGainDecibels (-6.0f);
    }

    //==============================================================================
    void setFrequency (Type newValue, bool force = false)
    {
        auto& osc = OscillatorProcessorChain.template get<oscIndex>();
        osc.setFrequency (newValue, force);
    }

    //==============================================================================
    void setLevel (Type newValue)
    {
        auto& gain = OscillatorProcessorChain.template get<gainIndex>();
        gain.setGainLinear (newValue); 
    }

    //==============================================================================
    void reset() noexcept
    {
         OscillatorProcessorChain.reset();
    }

    //==============================================================================
    template <typename ProcessContext>
    void process (const ProcessContext& context) noexcept
    {
        OscillatorProcessorChain.process (context);
    }

    //==============================================================================
    void prepare (const juce::dsp::ProcessSpec& spec)
    {
        OscillatorProcessorChain.prepare (spec);
    }

private:
    //==============================================================================
    enum
    {
        oscIndex,
        gainIndex
    };

    juce::dsp::ProcessorChain<juce::dsp::Oscillator<Type>, juce::dsp::Gain<Type>> OscillatorProcessorChain;
};
