/*
  ==============================================================================

    This file was auto-generated!

    It contains the basic framework code for a JUCE plugin processor.

  ==============================================================================
*/

#include "PluginProcessor.h"
#include "PluginEditor.h"

//==============================================================================
MyDspOscillatorGeneratorAudioProcessor::MyDspOscillatorGeneratorAudioProcessor()
     : AudioProcessor (BusesProperties().withOutput ("Output", AudioChannelSet::stereo(), true)),
       parameters (*this, nullptr, Identifier ("MyDspOscillatorGenerator"),
                        {
                            std::make_unique<AudioParameterFloat> ("osc1_gain",            // parameterID
                                                                   "Osc1 Gain",            // parameter name
                                                                   0.0f,              // minimum value
                                                                   1.0f,              // maximum value
                                                                   0.5f),             // default value
                            std::make_unique<AudioParameterFloat> ("osc1_freq",            // parameterID
                                                                   "Osc1 Freq",            // parameter name
                                                                   100.0f,              // minimum value
                                                                   5000.0f,              // maximum value
                                                                   432.0f),             // default value
                            std::make_unique<AudioParameterFloat> ("osc2_gain",            // parameterID
                                                                   "Osc2 Gain",            // parameter name
                                                                   0.0f,              // minimum value
                                                                   1.0f,              // maximum value
                                                                   0.5f),             // default value
                            std::make_unique<AudioParameterFloat> ("osc2_freq",            // parameterID
                                                                   "Osc2 Freq",            // parameter name
                                                                   100.0f,              // minimum value
                                                                   5000.0f,              // maximum value
                                                                   432.0f),             // default value
                            std::make_unique<AudioParameterFloat> ("global_gain",            // parameterID
                                                                   "Global Gain",            // parameter name
                                                                   0.0f,              // minimum value
                                                                   1.0f,              // maximum value
                                                                   0.5f)             // default value
           
                        })
{
    osc1GainParameter   = parameters.getRawParameterValue ("osc1_gain");
    osc1FreqParameter   = parameters.getRawParameterValue ("osc1_freq");
    osc2GainParameter   = parameters.getRawParameterValue ("osc2_gain");
    osc2FreqParameter   = parameters.getRawParameterValue ("osc2_freq");
    globalGainParameter = parameters.getRawParameterValue ("global_gain");
}

MyDspOscillatorGeneratorAudioProcessor::~MyDspOscillatorGeneratorAudioProcessor()
{
}

//==============================================================================
const String MyDspOscillatorGeneratorAudioProcessor::getName() const
{
    return JucePlugin_Name;
}

bool MyDspOscillatorGeneratorAudioProcessor::acceptsMidi() const
{
   #if JucePlugin_WantsMidiInput
    return true;
   #else
    return false;
   #endif
}

bool MyDspOscillatorGeneratorAudioProcessor::producesMidi() const
{
   #if JucePlugin_ProducesMidiOutput
    return true;
   #else
    return false;
   #endif
}

bool MyDspOscillatorGeneratorAudioProcessor::isMidiEffect() const
{
   #if JucePlugin_IsMidiEffect
    return true;
   #else
    return false;
   #endif
}

double MyDspOscillatorGeneratorAudioProcessor::getTailLengthSeconds() const
{
    return 0.0;
}

int MyDspOscillatorGeneratorAudioProcessor::getNumPrograms()
{
    return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
                // so this should be at least 1, even if you're not really implementing programs.
}

int MyDspOscillatorGeneratorAudioProcessor::getCurrentProgram()
{
    return 0;
}

void MyDspOscillatorGeneratorAudioProcessor::setCurrentProgram (int index)
{
}

const String MyDspOscillatorGeneratorAudioProcessor::getProgramName (int index)
{
    return {};
}

void MyDspOscillatorGeneratorAudioProcessor::changeProgramName (int index, const String& newName)
{
}

//==============================================================================
void MyDspOscillatorGeneratorAudioProcessor::prepareToPlay (double sampleRate, int samplesPerBlock)
{

    auto channels = static_cast<uint32> (jmin (getMainBusNumInputChannels(), getMainBusNumOutputChannels()));
    dsp::ProcessSpec spec { sampleRate, static_cast<uint32> (samplesPerBlock), channels };
    processorChain.prepare (spec);
    
    updateParameters();
    reset();
}

void MyDspOscillatorGeneratorAudioProcessor::releaseResources()
{
    // When playback stops, you can use this as an opportunity to free up any
    // spare memory, etc.
}


bool MyDspOscillatorGeneratorAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
{
    if (layouts.getMainOutputChannelSet() == AudioChannelSet::disabled())
        return false;
    
    if (layouts.getMainOutputChannelSet() != AudioChannelSet::mono()
     && layouts.getMainOutputChannelSet() != AudioChannelSet::stereo())
        return false;
 
    return true;
}


void MyDspOscillatorGeneratorAudioProcessor::updateParameters()
{
    //Update your parameters for your processes
    
    
    
    float osc1_freqHz = *osc1FreqParameter;
    float osc2_freqHz = *osc2FreqParameter;
    float osc1_amplitiude = *osc1GainParameter;
    float osc2_amplitiude = *osc2GainParameter;
    
    float global_amplitude = *globalGainParameter;;
    
    processorChain.get<osc1Index>().setFrequency (osc1_freqHz, true);
    processorChain.get<osc1Index>().setLevel (osc1_amplitiude);
    
    processorChain.get<osc2Index>().setFrequency (osc2_freqHz, true);
    processorChain.get<osc2Index>().setLevel (osc2_amplitiude);
    
    
    processorChain.get<masterGainIndex>().setGainLinear (global_amplitude);
    
}


void MyDspOscillatorGeneratorAudioProcessor::process(dsp::ProcessContextReplacing<float> context)
{

    processorChain.process (context);
}

void MyDspOscillatorGeneratorAudioProcessor::reset()
{
    processorChain.reset();
}





void MyDspOscillatorGeneratorAudioProcessor::processBlock (AudioBuffer<float>& buffer, MidiBuffer& midiMessages)
{
    ScopedNoDenormals noDenormals;
    auto totalNumInputChannels  = getTotalNumInputChannels();
    auto totalNumOutputChannels = getTotalNumOutputChannels();

    for (auto i = totalNumInputChannels; i < totalNumOutputChannels; ++i)
        buffer.clear (i, 0, buffer.getNumSamples());
    
    updateParameters();
    
    dsp::AudioBlock<float> block (buffer);
    process(dsp::ProcessContextReplacing<float> (block));
    
}

//==============================================================================
bool MyDspOscillatorGeneratorAudioProcessor::hasEditor() const
{
    return true; // (change this to false if you choose to not supply an editor)
}

AudioProcessorEditor* MyDspOscillatorGeneratorAudioProcessor::createEditor()
{
    return new MyDspOscillatorGeneratorAudioProcessorEditor (*this, parameters);
}

//==============================================================================
void MyDspOscillatorGeneratorAudioProcessor::getStateInformation (MemoryBlock& destData)
{
    auto state = parameters.copyState();
    std::unique_ptr<XmlElement> xml (state.createXml());
    copyXmlToBinary (*xml, destData);
}

void MyDspOscillatorGeneratorAudioProcessor::setStateInformation (const void* data, int sizeInBytes)
{
    std::unique_ptr<XmlElement> xmlState (getXmlFromBinary (data, sizeInBytes));

    if (xmlState.get() != nullptr)
        if (xmlState->hasTagName (parameters.state.getType()))
            parameters.replaceState (ValueTree::fromXml (*xmlState));
}

//==============================================================================
// This creates new instances of the plugin..
AudioProcessor* JUCE_CALLTYPE createPluginFilter()
{
    return new MyDspOscillatorGeneratorAudioProcessor();
}
